@@ -10,9 +10,10 @@
 # Copyright (c) 2019 Philipp Hörist <philipp@hoerist.com>
 # Copyright (c) 2020-2021 hippo91 <guillaume.peillex@gmail.com>
 # Copyright (c) 2021 Pierre Sassoulas <pierre.sassoulas@gmail.com>
+# Copyright (c) 2021 Marc Mueller <30130371+cdce8p@users.noreply.github.com>
 
 # Licensed under the LGPL: https://www.gnu.org/licenses/old-licenses/lgpl-2.1.en.html
-# For details: https://github.com/PyCQA/astroid/blob/master/LICENSE
+# For details: https://github.com/PyCQA/astroid/blob/main/LICENSE
 
 """Astroid hooks for the Python 2 GObject introspection bindings.
 
@@ -27,8 +28,10 @@ import re
 import sys
 import warnings
 
-from astroid import MANAGER, AstroidBuildingError, nodes
+from astroid import nodes
 from astroid.builder import AstroidBuilder
+from astroid.exceptions import AstroidBuildingError
+from astroid.manager import AstroidManager
 
 _inspected_modules = {}
 
@@ -207,7 +210,7 @@ def _import_gi_module(modname):
         except ImportError:
             astng = _inspected_modules[modname] = None
         else:
-            astng = AstroidBuilder(MANAGER).string_build(modcode, modname)
+            astng = AstroidBuilder(AstroidManager()).string_build(modcode, modname)
             _inspected_modules[modname] = astng
     else:
         astng = _inspected_modules[modname]
@@ -252,7 +255,7 @@ def _register_require_version(node):
     return node
 
 
-MANAGER.register_failed_import_hook(_import_gi_module)
-MANAGER.register_transform(
+AstroidManager().register_failed_import_hook(_import_gi_module)
+AstroidManager().register_transform(
     nodes.Call, _register_require_version, _looks_like_require_version
 )
