@@ -34,16 +34,19 @@ import os
 import pprint
 import types
 from functools import lru_cache
-from typing import Optional
+from typing import TYPE_CHECKING, Optional
 
 import astroid
-from astroid import context as contextmod
-from astroid import node_classes, util
+from astroid import util
+from astroid.context import InferenceContext, copy_context
 from astroid.exceptions import AttributeInferenceError, InferenceError, NoDefault
 from astroid.manager import AstroidManager
+from astroid.nodes import node_classes
 
 objects = util.lazy_import("objects")
 
+if TYPE_CHECKING:
+    from astroid.objects import Property
 
 IMPL_PREFIX = "attr_"
 
@@ -307,7 +310,7 @@ class FunctionModel(ObjectModel):
                         context=context,
                     )
 
-                context = contextmod.copy_context(context)
+                context = copy_context(context)
                 try:
                     cls = next(caller.args[0].infer(context=context))
                 except StopIteration as e:
@@ -449,7 +452,7 @@ class ClassModel(ObjectModel):
     @property
     def attr___bases__(self):
         obj = node_classes.Tuple()
-        context = contextmod.InferenceContext()
+        context = InferenceContext()
         elts = list(self._instance._inferred_bases(context))
         obj.postinit(elts=elts)
         return obj
@@ -469,7 +472,8 @@ class ClassModel(ObjectModel):
         thus it might miss a couple of them.
         """
         # pylint: disable=import-outside-toplevel; circular import
-        from astroid import bases, scoped_nodes
+        from astroid import bases
+        from astroid.nodes import scoped_nodes
 
         if not self._instance.newstyle:
             raise AttributeInferenceError(
@@ -749,8 +753,8 @@ class PropertyModel(ObjectModel):
 
     # pylint: disable=import-outside-toplevel
     def _init_function(self, name):
-        from astroid.node_classes import Arguments
-        from astroid.scoped_nodes import FunctionDef
+        from astroid.nodes.node_classes import Arguments
+        from astroid.nodes.scoped_nodes import FunctionDef
 
         args = Arguments()
         args.postinit(
@@ -771,7 +775,7 @@ class PropertyModel(ObjectModel):
 
     @property
     def attr_fget(self):
-        from astroid.scoped_nodes import FunctionDef
+        from astroid.nodes.scoped_nodes import FunctionDef
 
         func = self._instance
 
@@ -793,11 +797,11 @@ class PropertyModel(ObjectModel):
 
     @property
     def attr_fset(self):
-        from astroid.scoped_nodes import FunctionDef
+        from astroid.nodes.scoped_nodes import FunctionDef
 
         func = self._instance
 
-        def find_setter(func: objects.Property) -> Optional[astroid.FunctionDef]:
+        def find_setter(func: "Property") -> Optional[astroid.FunctionDef]:
             """
             Given a property, find the corresponding setter function and returns it.
 
